import copy

from graphviz import Digraph, nohtml
from IPython.display import display
from string import printable
import warnings

class Node:
    # if root node then previousNode will be None
    def __init__(self,identifier,values, valuesCountInNode,posInParentArray=None):
        assert all(isinstance(item, int) for item in values),"values has to be a int-list" 
        self.previousNode = None
        self.positionInParent_nextNodesArray = posInParentArray # easier to get siblings
        self.identifier = identifier # unique identifier
        self.values = values # e.g. Int-Array
        self.valuesCountInNode = valuesCountInNode
        self.nextNodes = [None] * (valuesCountInNode+1) # all children are firstly None

class BTree:
            
        
    def __init__(self, M):
        self.valuesCountInNode = M
        self.halffull = M//2 + (M%2)
        self.graph = Digraph('btree', 'dot', 
                             node_attr = {'shape': 'record', 
                                          'height': '.05',
                                          'fontsize': '10',
                                          'style': 'filled', 
                                          'fillcolor': '#FFFFFF'}, 
                             graph_attr = {'splines': 'line',
                                           'label':'M = '+str(M)})
        
        # all the node that were added
        self.nodeArray = []
        # if someone insert a node with an id
        #  that was already inserted -> throw an error
        self.identifierArray = []


    # if you used myBBaum.graph.render(filename='graph.dot')
    #  graphiz will save tree in dot-file format
    @staticmethod
    def loadFromDotFile(filepath):

        graphFile = open(filepath, "r")
        
        # following line is to get M value
        #  graph [label="M = 4" splines=line]
        M_value = graphFile.read().split("\"",2)[1].split(" ")[-1]
        newBtree = BTree(int(M_value))

        graphFile = open(filepath, "r")
        # add nodes to newBtree
        for line in graphFile:
            #print("line:",line)
            if "[label=\"<" in line:
                # is a node
                # we only need to split once because line looks like:
                #   0 [label="<f1> |18|<f2> |40|<f3> |63|<f4> |85|<f5> "]
                #   ^nodeName
                nodeName = line.split(" ", 1)[0]
                # delte all invisible characters
                nodeName = ''.join(c for c in nodeName if c.isprintable())
                valuesString = line.split("|")
                values = []
                for eventualVal in valuesString:
                    if eventualVal.isdigit():
                        values.append(int(eventualVal))

                newBtree.add_node(nodeName, values)

            elif ":" in line and "->" in line:
                # is an edge
                # strange code therefore look at the dotGraph source code
                #   0:f1 -> 1
                #   ^nodeName
                #   0:f1 -> 1
                # childNode ^
                #   0:f1 -> 1
                #      ^ atParentsPoint
                splitDot = line.split(":", 1)
                parentNode = splitDot[0]
                # delete all invisible characters
                parentNode = ''.join(c for c in parentNode if c.isprintable())
                #print("parentNode:",parentNode)
                childNode = line.split(" ")[-1]
                #delte all invisible characters
                childNode = ''.join(c for c in childNode if c.isprintable())
                #print("childNode:", childNode)
                # [1:] delte the first character which is a 'f'
                atParentsPoint = int(splitDot[1].split(" ", 1)[0][1:])

                #print("atParentsPoint:", atParentsPoint)

                newBtree.add_edge(parentNode, childNode, atParentsPoint)

        return newBtree


    # this method takes the node-datastructure form
    #  the old tree and build a new tree based on this
    #  node-datastructure
    @staticmethod
    def updateGraph(oldBtree):
        newTree = BTree(oldBtree.valuesCountInNode)
        #add nodes
        for node in oldBtree.nodeArray:
            newTree.add_node(node.identifier, node.values)
            curNode = newTree.getNode(node.identifier)
            # ugly bufix workaround
            curNode.nextNodes = [None]*len(node.nextNodes)

        # add edges
        for node in oldBtree.nodeArray:
            for i,child in enumerate(node.nextNodes):
                if child != None:
                    #print("length of node",len(node.nextNodes))
                    newTree.add_edge(node.identifier,child.identifier,i+1)

        return newTree

        
    def add_node(self, name, elements):   
        i = 1
        res_str = "<f" + str(i) + "> "

        for x in elements:      
            if not isinstance(x, int):
                print(str(x) + ' should be an integer.')
                     
            i = i + 1
            append_str = "|" + str(x) + "|<f" + str(i) + "> "
            res_str = res_str + append_str
        
        self.graph.node(name, nohtml(res_str))
        
        
        if len(self.identifierArray) == 0:
            #root node
            self.nodeArray = [Node(name,elements,self.valuesCountInNode)]
            self.identifierArray.append(name)
        elif name in self.identifierArray:
            assert False, "Bitte nutze einen anderen Name für diese Node."
        else:
            self.nodeArray.append(Node(name,elements,self.valuesCountInNode))
            self.identifierArray.append(name)

    
    def add_edge(self, parent, child, n_child):       
        self.graph.edge(parent + ':f' + str(n_child), child)

        #search both nodes and add their relations
        parentNode = self.getNode(parent)
        childNode = self.getNode(child)
 
        assert parentNode != None and childNode != None, "internal bug: could not find parent or child node in nodeArray."
        
        # add in previous or next node array
        childNode.previousNode = parentNode
        
        isNotInNextArray = next((x for x in parentNode.nextNodes if x != None and childNode.identifier == x.identifier), None) == None
        if isNotInNextArray:
            #index of the node in nextnodes determindes how the tree locks like
            #print("parent:",parent,"child:",child,"n_child:",n_child)
            if n_child-1 >= len(parentNode.nextNodes):
                parentNode.nextNodes.insert(n_child-1,childNode)
            else:
                parentNode.nextNodes[n_child - 1] = childNode
            childNode.positionInParent_nextNodesArray = n_child-1

    def getNode(self,nodeID):
        return next((x for x in self.nodeArray if x.identifier == nodeID), None)

    def getRootNode(self):
        # if there are two rootNodes return None
        rootNodes = []
        for node in self.nodeArray:
            if node.previousNode == None:
                rootNodes.append(node)
        if len(rootNodes) != 1:
            return None
        return rootNodes[0]

    def delteNode(self,nodeID):
        node = next((x for x in self.nodeArray if nodeID == x.identifier), None)
        if not nodeID in self.identifierArray or node == None:
            warnings.warn("Node ID: "+nodeID+" does not exist.")
            return

        #delte from own datastructure
        self.identifierArray.remove(nodeID)
        for i,tmpNode in enumerate(self.nodeArray):
            if tmpNode.identifier == nodeID:
                self.nodeArray.pop(i)
                continue
            #delte if node was parent
            if tmpNode.previousNode != None\
                and tmpNode.previousNode.identifier == nodeID:
                tmpNode.previousNode = None
            #delete if node is next node
            for i,child in enumerate(tmpNode.nextNodes):
                if child != None and child.identifier == nodeID:
                    tmpNode.nextNodes[i] = None


        #delete from graphviz data structure
        #  easy trick: just create the newTree we are having NOW
        #  and set it to self
        #self = BTree.updateGraph(self)
        # IMPORTANT but do it in upper function call

    def valueIsInBbaum(self,value):
        for node in self.nodeArray:
            if value in node.values:
                return True
        return False

    @staticmethod
    def isLeafNode(node):
        return node.nextNodes.count(None) == len(node.nextNodes)

    @staticmethod
    def getSibling(currentNode, bool_getLeft):
        if currentNode.previousNode == None:
            return None

        # save in variable because I chose a too long name
        indexNextNodes = currentNode.positionInParent_nextNodesArray

        if indexNextNodes == None:
            # should ... never be the case...
            warnings.warn("WARNING: positionInParent_nextNodesArray is None!")
            return None

        if bool_getLeft:
            if indexNextNodes == 0:
                return None

            return currentNode.previousNode.nextNodes[indexNextNodes - 1]
        else:
            if indexNextNodes == len(currentNode.previousNode.nextNodes) - 1:
                return None

            return currentNode.previousNode.nextNodes[indexNextNodes + 1]

    # its easier for students if they can easily copy the generate graph text
    # in order to make the exercsie
    @staticmethod
    def generateCopyText(node, text, treeName):
        if node == None:
            return ""
        tmpText = treeName + ".add_node(\'" + \
                  str(node.identifier) + "\', " + str(node.values) + ")\n"
        for i, child in enumerate(node.nextNodes):
            if child != None:
                tmpText += BTree.generateCopyText(child, text, treeName)
                tmpText += treeName + ".add_edge(\'" + \
                           node.identifier + "\', \'" + child.identifier + \
                           "\', " + str(i + 1) + ")\n"
        return text + tmpText

    def draw(self):
        display(self.graph)
    
    




#import sys
#import os
# to import from parent folder
## sets current folder to parent folder
#current = os.path.dirname(os.path.realpath(__file__))
#parent = os.path.dirname(current)
#sys.path.append(parent)

from .BBaum_Creator import BTree_Creator
from .BBaum_Creator import TreeTypes
from .BBaum import BTree
import numpy as np
import random
import warnings

class BTree_Ex_Generator:

    pathToValues = "values.txt"
    pathToGraph = "graph.dot"

    def __init__(self, operationCount, treeType, bbaum=None,
                        values=None, # <- this values do not have a correction checkt!
                        M=4,height=1,numbersUntil=100):

        self.treeType = treeType
        # create insert tree
        if bbaum == None:
            self.bbaum = \
                BTree_Creator(height, numbersUntil, M, treeType).myBBaum
        else:
            self.bbaum = bbaum

        self.operationsAndSol = [] #array of the class BTree_OneOp
        self.howManyOperations = self.checkOperationCount(operationCount,treeType==TreeTypes.INSERT)
        #init both arrays: deleteOperationsAndSol and insertOperationsAndSol

        self.generateOperationValues(values)

        self.copyPastTextForTree = BTree.generateCopyText(
                                    self.bbaum.getRootNode(),"","b")


    def saveArrayToDisk(self,array,toPath):
        out = ""
        for i,elm in enumerate(array):
            out += str(elm)
            if i < len(array) - 1:
                out += " "

        f = open(toPath, "w")
        f.write(out)
        f.close()

    def readArrayFromDisk(self,fromPath):
        f = open(fromPath, "r")
        strArray = f.read().split(" ")
        return [int(x) for x in strArray]



    def generateOperationValues(self, values):

        if self.howManyOperations > 0:

            if values != None:
                #insertValues = self.readArrayFromDisk(pathtoValues)
                self.operationsAndSol = values
            else:

                leafNodes = [node for node in self.bbaum.nodeArray if
                             node.nextNodes.count(None) == len(node.nextNodes)]
                self.operationsAndSol = self.createOperationValues(leafNodes)
                #self.saveArrayToDisk( [x.operation for x in self.insertOperationsAndSol], self.pathToValues)

    def createOperationValues(self,leafNodes):
        tmpOperationValues = []

        allValuesOfTheTree = []
        for node in leafNodes:
            for value in node.values:
                allValuesOfTheTree.append(value)
        allValuesOfTheTree.sort()
        #print("type",str(self.treeType)==str(TreeTypes.INSERT_TRIVIAL))
        if str(self.treeType) == str(TreeTypes.INSERT) \
            or str(self.treeType) == str(TreeTypes.INSERT_TRIVIAL):
            if str(self.treeType) == str(TreeTypes.INSERT):
                    # first insertion is in a full leaf
                    fullLeaf = next((x for x in leafNodes if len(x.values) == self.bbaum.valuesCountInNode), None)
                    if fullLeaf != None:
                        value = int(np.random
                                    .random_integers(low=fullLeaf.values[0],
                                                     high=fullLeaf.values[-1]))
                        tmpOperationValues.append(value)
                        self.howManyOperations -= 1
                    else:
                        # no full leafNode in Tree
                        warnings.warn("Was not able to ask for insertion in full leaf.")
            else:
                #is trivial
                notFullLeaf = next((x for x in leafNodes if len(x.values) < self.bbaum.valuesCountInNode), None)
                if notFullLeaf != None:
                    value = int(np.random
                                .random_integers(low=notFullLeaf.values[0],
                                                 high=notFullLeaf.values[-1]))
                    tmpOperationValues.append(value)
                    self.howManyOperations -= 1
                else:
                    # no full leafNode in Tree
                    warnings.warn("Was not able to ask for insertion in a not full leaf.")
            # other can be random
            minVal = allValuesOfTheTree[0]
            maxVal = allValuesOfTheTree[-1]
            while self.howManyOperations > 0:
                value = int(np.random
                            .random_integers(low=minVal,
                                             high=maxVal))
                tmpOperationValues.append(value)
                self.howManyOperations -= 1

        elif str(self.treeType) == str(TreeTypes.DELETE_VERSCHMELZ)\
            or str(self.treeType) == str(TreeTypes.DELETE_AUSGLEICH)\
            or str(self.treeType) == str(TreeTypes.DELETE_TRIVIAL):
            print("delete")
            # ---------------  ensure augleich and verschmelzung  ---------------
            # tree is an deletion Tree
            #  search for ausgleich situation:
            #      none halffull halffull+1  or  halffull halffull halffull+1
            if not str(self.treeType) == str(TreeTypes.DELETE_TRIVIAL):
                (ausgleichLeaf,
                 verschmelzLeaf) = self.findAusgleichVerschmelzLeaf(leafNodes)

                for i, node in enumerate([ausgleichLeaf, verschmelzLeaf]):
                    if node == None:
                        if i == 0:
                            if str(self.treeType) == TreeTypes.DELETE_AUSGLEICH:
                                warnings.warn("Was not able to ensure ausgleich operation")
                                break
                        else:
                            if str(self.treeType) == TreeTypes.DELETE_VERSCHMELZ:
                                warnings.warn("Was not able to ensure verschmelz operation")
                                break
                        continue

                    if i >= len(node.values):
                        # the tree has just 1 value per node
                        warnings.warn("The tree has too less values in one node.")
                        break

                    if node != None and self.howManyOperations > 0:
                        index = int(np.random
                                    .random_integers(low=0,
                                                     high=len(node.values) - 1))
                        # this loop is important
                        stopInfinityLoop = 10
                        while not node.values[index] in allValuesOfTheTree \
                                and stopInfinityLoop > 0:
                            index = int(np.random
                                        .random_integers(low=0,
                                                         high=len(node.values) - 1))
                            stopInfinityLoop -= 1
                        if stopInfinityLoop == 0:
                            break

                        tmpOperationValues.append(node.values[index])

                        # delete from allValuesOfTheTree array
                        allValuesOfTheTree.remove(node.values[index])
                        self.howManyOperations -= 1
            else:
                #is trivial
                notHalfFullLeaf = next((x for x in leafNodes if len(x.values) > self.bbaum.halffull), None)
                if notHalfFullLeaf != None:
                    value = random.choice(notHalfFullLeaf.values)
                    tmpOperationValues.append(value)

                    # delete from allValuesOfTheTree array
                    allValuesOfTheTree.remove(value)
                    self.howManyOperations -= 1
                else:
                    # no full leafNode in Tree
                    warnings.warn("Was not able to ask for trivial deltion.")
            # ------------------------------------------------------------

            # other can be random
            while len(allValuesOfTheTree) > 0 and self.howManyOperations > 0:
                index = 0
                if len(allValuesOfTheTree) > 1:
                    index = int(np.random
                                .random_integers(low=0,
                                                 high=len(allValuesOfTheTree) - 1))
                tmpOperationValues.append(allValuesOfTheTree[index])
                # delete from allValuesOfTheTree array
                allValuesOfTheTree.pop(index)
                self.howManyOperations -= 1

        return tmpOperationValues


    #helper function for finding delting leaf
    def findAusgleichVerschmelzLeaf(self, leafNodes):
        ausgleichLeaf = None
        verschmelzLeaf = None
        for i, node in enumerate(leafNodes):
            leftSibling = BTree.getSibling(node, True)
            leftHalffull = True
            leftMoreThanHalffull = False
            if leftSibling != None:
                leftHalffull = len(leftSibling.values) == self.bbaum.halffull
                leftMoreThanHalffull = len(leftSibling.values) > self.bbaum.halffull

            rightSibling = BTree.getSibling(node, False)
            rightHalffull = True
            rightMoreThanHalffull = False
            if rightSibling != None:
                rightHalffull = len(rightSibling.values) == self.bbaum.halffull
                rightMoreThanHalffull = len(rightSibling.values) > self.bbaum.halffull

            if len(node.values) == self.bbaum.halffull:
                # first search for ausgleich
                if ausgleichLeaf == None and (leftMoreThanHalffull or rightMoreThanHalffull):
                    ausgleichLeaf = node
                if verschmelzLeaf == None and (leftHalffull and rightHalffull):
                    verschmelzLeaf = node
        return (ausgleichLeaf, verschmelzLeaf)




    #if there are less nodes than you want to delete than in the tree
    #  -> try to avoid that situation
    def checkOperationCount(self, operCount, isInsertingTree):
        if operCount <= 0:
            return 0
        if not isInsertingTree:
            leafNodes = [node for node in self.bbaum.nodeArray if node.nextNodes.count(None) == len(node.nextNodes)]
            countLeafNodeValues = 0
            for node in leafNodes:
                countLeafNodeValues += sum(x is not None for x in node.values)

            if operCount > countLeafNodeValues:
                return countLeafNodeValues
        return operCount


    def generateText(self):
        operations = [x for x in self.operationsAndSol]

        return str(self.treeType)+": "+str(operations)






#import sys
#import os
# to import from parent folder
## sets current folder to parent folder
#current = os.path.dirname(os.path.realpath(__file__))
#parent = os.path.dirname(current)
#sys.path.append(parent)

from .BBaum import BTree
import numpy as np
from enum import Enum

class TreeTypes(Enum):
    INSERT = "INSERT"
    INSERT_TRIVIAL = "INSERT_TRIVIAL"
    DELETE_VERSCHMELZ = "DELETE_VERSCHMELZ"
    DELETE_AUSGLEICH = "DELETE_AUSGLEICH"
    DELETE_TRIVIAL = "DELETE_TRIVIAL"

class BTree_Creator:
    def __init__(self, height, maxValueInTree,
                 valuesCountInOneNode, treeType):
        self.height = height
        # values from 0 to maxValueInTree
        self.maxValueInTree = maxValueInTree
        # maxValuesInOneNode is the M value from the lecture
        self.valuesCountInOneNode = valuesCountInOneNode
        # because we dont want to build trivial trees
        # we disguingish between imserting and deleting trees
        self.treeType = treeType
        self.isInsertingTree = treeType == TreeTypes.INSERT
        # while correction we dont want an endless loop
        self.maxInterationsCorrection = 12
        # to ensure that not only NOT trival inserting or deleting exist
        #  counts how many leaf nodes will be skiped


        # just for safetiness check if tree is 100% correct
        treeIncorrect = True
        while self.maxInterationsCorrection > 0 and treeIncorrect:
            # HERE is btree instance
            self.delayForInsertDeleteLeaf = self.determineLeafForOperations()
            self.myBBaum = BTree(valuesCountInOneNode)
            print("creating tree...")
            self.buildTree([(0, None, 0, maxValueInTree, 0)])
            print("done.")
            print("checking if tree is correct...")
            rootNode = next((x for x in self.myBBaum.nodeArray
                             if x.previousNode == None), None)
            treeIncorrect = BTree_Creator\
                            .treeIncorrect(self.myBBaum, rootNode,self.valuesCountInOneNode)
            if not treeIncorrect:
                print("done.")
            else:
                print("not correct.")
            self.maxInterationsCorrection -= 1
        assert self.maxInterationsCorrection > 0, "Please restart program."


    # which note should be the note for non-trivial deleting and inserting
    def determineLeafForOperations(self):
        randomIndex = 0
        if self.valuesCountInOneNode > 1:
            randomIndex = int(np.random
                          .random_integers(low=0,
                                           high=self.valuesCountInOneNode-1))
        return randomIndex



    # ---------------- check if tree is correct ----------------
    @staticmethod
    def treeIncorrect(bbaum,parentNode,valuesCountInOneNode):
        if parentNode.nextNodes == [None] * (valuesCountInOneNode + 1):
            return False
        elif parentNode.nextNodes.count(None) != 0:
            #"Tree has to be full! every inner node has to have a child"
            return True
        # one of the children is not None
        isIncorrect = False
        for i, child in enumerate(parentNode.nextNodes):
            if i < len(parentNode.values):
                # because last one has to be compered differently
                if child != None:
                    # all values has to be smaller
                    if child.values[-1] >= parentNode.values[i]:
                        # last is bigger -> btree property hurted
                        print("Tree is incorrect.")
                        print("at nodeNumber:", i, "parentValues:",
                              parentNode.values, "childValues:",
                              child.values)
                        return True
                    else:
                        return isIncorrect or BTree_Creator.treeIncorrect(bbaum,child,valuesCountInOneNode)
            else:
                # now check if its bigger...
                if child != None:
                    if child.values[0] < parentNode.values[i - 1]:
                        # last is smaller -> btree property hurted
                        print("Tree is incorrect.")
                        print("at nodeNumber:", i, "parentValues:",
                              parentNode.values, "childValues:",
                              child.values)
                        return True
                    else:
                        return isIncorrect or BTree_Creator.treeIncorrect(bbaum,child)

        return isIncorrect

    # build recusivly use level order because of naming the nodes
    # min and max value determine how which values will be coveres and this
    # node and its decand
    # nodename is a number
    # parentNode for connecting
    def buildTree(self, stackForRecursion):

        # here is somehow a bug maybe not everything get loaded
        # if stack is NOT empty
        if len(stackForRecursion) == 0:
            return

        (nodeName, parentNode, minValue, maxValue,
         currentHeight) = stackForRecursion.pop(0)

        if currentHeight > self.height:
            return

        isLeftChild = True

        if parentNode != None:
            isLeftChild = maxValue <= parentNode

        values = self.createValueArray(minValue, maxValue,
                                       isLeftChild, currentHeight)

        if values == None:
            return

        # print("Nodename:",nodeName, self.getNodeName(nodeName))
        self.myBBaum.add_node(BTree_Creator.getNodeName(nodeName), values)

        if parentNode != None:
            thisNodeIsChildNumber = (nodeName % (self.valuesCountInOneNode + 1))
            if thisNodeIsChildNumber == 0:
                thisNodeIsChildNumber = self.valuesCountInOneNode + 1
            # print("add edge from:",parentNode, "to", nodeName,
            #       "N:",thisNodeIsChildNumber)

            self.myBBaum.add_edge(BTree_Creator.getNodeName(parentNode),
                                  BTree_Creator.getNodeName(nodeName),
                                  thisNodeIsChildNumber)

        # :) calculate the nodeposition in btree
        nextNodeName = nodeName * (self.valuesCountInOneNode + 1)

        borders = values.copy()
        borders.append(maxValue)
        borders.insert(0, minValue)

        for i in range(1, len(borders)):
            previousIntervallBorder = borders[i - 1]
            nextIntervallBorder = borders[i]
            nextNodeName += 1
            # print("added next node:",nextNodeName, self.getNodeName(nextNodeName))
            nodeTupel = (nextNodeName, nodeName,
                         previousIntervallBorder,
                         nextIntervallBorder, currentHeight + 1)

            stackForRecursion.append(nodeTupel)

        # rekursiv call
        for _ in range(0, self.valuesCountInOneNode + 1):
            self.buildTree(stackForRecursion)

    # ---------------- create values in one node ----------------
    def createValueArray(self, minVal, maxVal, isLeftChild,
                         currentHeight):

        distance = maxVal - minVal

        if distance == 1:
            if isLeftChild:
                return [minVal] * self.valuesCountInOneNode
            else:
                return [maxVal] * self.valuesCountInOneNode
        if distance <= 0:
            return None

        stepSize = distance // (self.valuesCountInOneNode)

        values = []
        curMin = minVal + 1
        curMax = curMin - 1 + stepSize

        fillSizeOfNodes = self.determineFillSize(currentHeight)

        for i in range(0, fillSizeOfNodes):
            low = curMin + (stepSize // 6)
            high = curMax - (stepSize // 6)
            if low >= high:
                low = curMin
                high = curMax
                if low >= high:
                    return None
                    #raise WrongInputVariables(
                        #"Die range für diesen Unterknoten besteht nur aus einer Zahl. Wir möchten diese Bäume vermeiden.\n 1. run script again\n 2. Bitte starte Script erneut mit anderen Height- und/oder maxValue-Werten.")

            newValue = int(np.random
                           .random_integers(low=low,
                                            high=high))
            if i > 0 and newValue < values[i - 1]:
                # now every value is sorted!
                newValue = values[i - 1]

            values.append(newValue)

            curMin = curMax
            curMax = curMin + stepSize

        return values

    def determineFillSize(self,currentHeight):
        fillSizeOfNodes = int(np.random
                              .random_integers(low=self.myBBaum.halffull,
                                               high=self.valuesCountInOneNode))
        if currentHeight == self.height:
            # we want neither one leaf full nor one leaf half full
            if self.isInsertingTree:
                if self.delayForInsertDeleteLeaf == 0:
                    if str(self.treeType) == str(TreeTypes.INSERT):
                        fillSizeOfNodes = self.valuesCountInOneNode
                    elif str(self.treeType) == str(TreeTypes.INSERT_TRIVIAL):
                        fillSizeOfNodes = self.valuesCountInOneNode - 1

            else:
                if self.treeType == TreeTypes.DELETE_TRIVIAL:
                    if self.delayForInsertDeleteLeaf == 0:
                        self.fillSizeOfNodes = self.valuesCountInOneNode
                else:
                    if self.delayForInsertDeleteLeaf == 1:
                        fillSizeOfNodes = self.myBBaum.halffull
                    elif self.delayForInsertDeleteLeaf == 0:
                        fillSizeOfNodes = self.myBBaum.halffull
                    elif self.delayForInsertDeleteLeaf == -1:
                        # second leaf half many +1 nodes -> ensures Ausgleich
                        # second leaf half many nodes -> ensures Verschmelzen
                        puls = 1
                        if str(self.treeType) == str(TreeTypes.DELETE_VERSCHMELZ):
                            plus = 0
                        fillSizeOfNodes = self.myBBaum.halffull + puls

            self.delayForInsertDeleteLeaf -= 1
        return fillSizeOfNodes

    # insert a number and it return the letter combination
    # for ex: 2=B,3=C,27=AA
    @staticmethod
    def getNodeName(num):
        numOfA_s = num // 26
        nodeName = ''
        for _ in range(0, numOfA_s):
            nodeName += 'A'

        return nodeName + str(chr((num % 26) + ord('A')))

    # insert a number and it return the letter combination
    # for ex: B=2,C=3,AA=27
    @staticmethod
    def convertNodeNameInNumber(name):
        number = -ord('A')
        for char in name:
            number += ord(char)

        return number


from .BBaum_Creator import BTree_Creator
from .BBaum_Creator import WrongInputVariables
import sys

height = 1
maxValueInTree = 100
valuesCountInOneNode = 4
isInsertionTree = True

def checkInput():
    global height
    global maxValueInTree
    global valuesCountInOneNode
    global isInsertionTree

    if len(sys.argv) > 1:
        if (len(sys.argv) - 1) % 2 != 0:
            return 1

        flagsCount = (len(sys.argv) - 1) // 2

        if '-h' in sys.argv:
            index = sys.argv.index('-h')
            height = int(sys.argv[index + 1])
            flagsCount -= 1
        elif '--height' in sys.argv:
            index = sys.argv.index('--height')
            height = int(sys.argv[index + 1])
            flagsCount -= 1

        if '--valueRange' in sys.argv:
            index = sys.argv.index('--valueRange')
            maxValueInTree = int(sys.argv[index + 1])
            flagsCount -= 1
        elif '-v' in sys.argv:
            index = sys.argv.index('-v')
            maxValueInTree = int(sys.argv[index + 1])
            flagsCount -= 1

        if '--countValuesInOneNode' in sys.argv:
            index = sys.argv.index('--countValuesInOneNode')
            valuesCountInOneNode = int(sys.argv[index + 1])
            flagsCount -= 1
        elif '-c' in sys.argv:
            index = sys.argv.index('-c')
            valuesCountInOneNode = int(sys.argv[index + 1])
            flagsCount -= 1

        if '-t' in sys.argv:
            index = sys.argv.index('-t')
            height = bool(sys.argv[index + 1])
            flagsCount -= 1
        elif '--isInsertingTree' in sys.argv:
            index = sys.argv.index('--isInsertingTree')
            height = bool(sys.argv[index + 1])
            flagsCount -= 1

        if flagsCount != 0:
            return 1

    return 0

def main():
    # standart values
    global height
    global maxValueInTree
    global valuesCountInOneNode
    global isInsertionTree

    errorCode = checkInput()

    if errorCode != 0:
        return 1

    tree = BTree_Creator(height, maxValueInTree,
                         valuesCountInOneNode,
                         isInsertionTree)
    
    tree.myBBaum.graph.save()
    tree.myBBaum.graph.render(filename='graph', view=True, cleanup=1)

    return 0

if __name__ == "__main__":
    errorCode = main()
    if errorCode == 1:
        print("usages: python3 -m dbis-bbaum [-flag value]")
        print("flags:")
        print("--height | -h [interger >= 0]   for example:  --height 2")
        print("--valueRange | -v [interger > 0]   for example:  --valueRange 100")
        print("--countValuesInOneNode | -c [interger > 0]   for example:  --countValuesInOneNode 4")
        print("--isInsertingTree | -t [Boolean]   for example:  --isInsertingTree True")



